@@ -50,13 +50,15 @@ class Control:
     # To be defined in subclasses as an iterable, if necessary
     possible_inputs = None
 
-    def __init__(self,
-                 range_limits,
-                 round_digits=3,
-                 print_log=False,
-                 save_log=True,
-                 log_file='Control_Log.txt',
-                 savepath='.'):
+    def __init__(
+        self,
+        range_limits,
+        round_digits=3,
+        print_log=False,
+        save_log=True,
+        log_file='Control_Log.txt',
+        savepath='.',
+    ):
         """Init control object, with parameters:
 
         - range_limits: limits in settable parameter value (tuple min, max)
@@ -80,7 +82,7 @@ class Control:
 
     # -------- Private methods for class operation behind the scenes ---------
 
-    def _check_range_limits(self, qty, value):
+    def _check_range_limits(self, qty, value, message=True):
         """Return value if within limits, else return higher or lower limit."""
         vmin, vmax = self.range_limits
         value_min = vmin if vmin is not None else -inf
@@ -93,10 +95,11 @@ class Control:
                 value_setpt = value_min
             else:
                 value_setpt = value_max
-            msg = (f'Required {qty}={value} outside of allowed '
-                   f'range {value_min}-{value_max}. '
-                   f'Setting kept at {value_setpt}.')
-            self._manage_message(msg)
+            if message:
+                msg = (f'Required {qty}={value} outside of allowed '
+                       f'range {value_min}-{value_max}. '
+                       f'Setting kept at {value_setpt}.')
+                self._manage_message(msg)
             return value_setpt
 
     def _print_ramp_info(self, qty, v1, v2, duration):
@@ -164,29 +167,51 @@ class Control:
 
     # ------------- Factory methods to generate program objects --------------
 
-    def program(self, repeat=1, **steps):
+    def program(
+        self,
+        repeat=1,
+        **steps,
+    ):
         """Convenience method to generate a program. see Program Class."""
-        return Program(control=self,
-                       repeat=repeat,
-                       **steps)
+        return Program(
+            control=self,
+            repeat=repeat,
+            **steps,
+        )
 
-    def stairs(self, duration=None, repeat=1, **steps):
+    def stairs(
+        self,
+        duration=None,
+        repeat=1,
+        **steps,
+    ):
         """Convenience method to generate a stairs program. see Stairs Class."""
-        return Stairs(control=self,
-                      duration=duration,
-                      repeat=repeat,
-                      **steps)
+        return Stairs(
+            control=self,
+            duration=duration,
+            repeat=repeat,
+            **steps,
+        )
 
-    def teeth(self, slope=None, slope_unit='/min', plateau_duration=None,
-              start='plateau', repeat=1, **steps):
+    def teeth(
+        self,
+        slope=None,
+        slope_unit='/min',
+        plateau_duration=None,
+        start='plateau',
+        repeat=1,
+        **steps,
+    ):
         """Convenience method to generate a teeth program. see Teeth Class."""
-        return Teeth(control=self,
-                     slope=slope,
-                     slope_unit=slope_unit,
-                     plateau_duration=plateau_duration,
-                     start=start,
-                     repeat=repeat,
-                     **steps)
+        return Teeth(
+            control=self,
+            slope=slope,
+            slope_unit=slope_unit,
+            plateau_duration=plateau_duration,
+            start=start,
+            repeat=repeat,
+            **steps,
+        )
 
 
 # ----------------------------------------------------------------------------
@@ -197,14 +222,16 @@ class Control:
 class PeriodicControl(Control):
     """Control using periodic update of device setting, no feedback"""
 
-    def __init__(self,
-                 dt=1,
-                 range_limits=(None, None),
-                 round_digits=3,
-                 print_log=False,
-                 save_log=True,
-                 log_file='Control_Log.txt',
-                 savepath='.'):
+    def __init__(
+        self,
+        dt=1,
+        range_limits=(None, None),
+        round_digits=3,
+        print_log=False,
+        save_log=True,
+        log_file='Control_Log.txt',
+        savepath='.',
+    ):
         """Create PeriodicControl object, with parameters:
 
         - dt: time interval (s) between commands
@@ -218,13 +245,15 @@ class PeriodicControl(Control):
                     (default: Control_Log.txt)
         - savepath: directory in which to save the log file
         """
-        Control.__init__(self,
-                         range_limits=range_limits,
-                         round_digits=round_digits,
-                         print_log=print_log,
-                         save_log=save_log,
-                         log_file=log_file,
-                         savepath=savepath)
+        Control.__init__(
+            self,
+            range_limits=range_limits,
+            round_digits=round_digits,
+            print_log=print_log,
+            save_log=save_log,
+            log_file=log_file,
+            savepath=savepath,
+        )
         self._dt = dt
         self.timer = oclock.Timer(interval=dt)
         self.stop_event = Event()
@@ -297,9 +326,9 @@ class PeriodicControl(Control):
         final_setting = self._check_range_limits(qty, target_setting)
         try:
             self.apply_setting(final_setting)
-        except Exception:
+        except Exception as e:
             t_str = datetime.now().isoformat(sep=' ', timespec='seconds')
-            print(f'Impossible to apply setting {qty}={value} ({t_str}).')
+            print(f'Impossible to apply setting {qty}={value} ({t_str}).\n{e}')
         else:
             self.print_setting(final_setting)
 
@@ -315,12 +344,13 @@ class PeriodicControl(Control):
 
     # ============================ Ramping methods ===========================
 
-    def _apply_setting_and_check_done(self, qty, value, attempts=5):
+    def _apply_setting_and_check_done(self, qty, value, attempts=10):
         """Stay at a given value setting for the quantity of interest (blocking)
 
         Not a public method; is used by _ramp() and ramp().
         """
-        target_setting = self._convert_input(**{qty: value})
+        setting = self._convert_input(**{qty: value})
+        target_setting = self._check_range_limits(qty, setting, message=False)
 
         for _ in range(attempts):
 
@@ -339,9 +369,10 @@ class PeriodicControl(Control):
             self.timer.checkpt()
 
         else:
-            print(f'WARNING -- Could not apply setting: '
-                  f'target setting {target_round} and actual setting '
-                  f'{actual_setting} do not match.')
+            msg = 'WARNING -- Could not apply setting: '
+            msg += f'target setting {target_round} and actual setting '
+            msg += f'{actual_setting} do not match.'
+            self._manage_message(msg, force_print=True)
 
     def _ramp(self, duration, **values):
         """Ramp from val1 to val2 with given duration.
@@ -425,18 +456,18 @@ class PeriodicControl(Control):
 class RecordingControl(PeriodicControl):
     """Control of prevo Recordings objects"""
 
-    possible_inputs = 'values',
-
-    def __init__(self,
-                 recording=None,
-                 ppty=None,
-                 dt=1,
-                 range_limits=(None, None),
-                 round_digits=3,
-                 print_log=False,
-                 save_log=True,
-                 log_file='Control_Log_Recording.txt',
-                 savepath='.'):
+    def __init__(
+        self,
+        recording=None,
+        ppty=None,
+        dt=1,
+        range_limits=(None, None),
+        round_digits=3,
+        print_log=False,
+        save_log=True,
+        log_file='Control_Log_Recording.txt',
+        savepath='.',
+    ):
         """Create PeriodicTemperatureControl object, with parameters:
 
         - recording: Recording object or subclass.
@@ -450,20 +481,24 @@ class RecordingControl(PeriodicControl):
         - save_log: if True (default), save succession of settings sent to
                     device into a .txt file
         - log_file: name of .txt file in which to save log of settings.
-                    (default: Control_Log.txt)
+                    (default: Control_Log_Recording.txt)
         - savepath: directory in which to save the log file
         """
-        PeriodicControl.__init__(self,
-                                 dt=dt,
-                                 range_limits=range_limits,
-                                 round_digits=round_digits,
-                                 print_log=print_log,
-                                 save_log=save_log,
-                                 log_file=log_file,
-                                 savepath=savepath)
+        self.ppty = ppty
+        self.possible_inputs = self.ppty.commands
+
+        PeriodicControl.__init__(
+            self,
+            dt=dt,
+            range_limits=range_limits,
+            round_digits=round_digits,
+            print_log=print_log,
+            save_log=save_log,
+            log_file=log_file,
+            savepath=savepath,
+        )
 
         self.recording = recording
-        self.ppty = ppty
 
     def _apply_setting(self, value):
         """Set property value on recording."""
@@ -480,4 +515,11 @@ class RecordingControl(PeriodicControl):
         return f'Setting: {self.ppty.attribute}={value}'
 
     def _convert_input(self, **values):
-        return values['values']
+        for command in self.ppty.commands:
+            try:
+                return values[command]
+            except KeyError:
+                pass
+        else:
+            input_key, = values.keys()
+            raise ValueError(f'Input {input_key} does not match {self.ppty.commands}.')
